! Copyright Jan Skowron 2012, 2016
! for internal use and educational purposes
! please do not redistribute (point to http://www.astrouw.edu.pl/~jskowron/ulens/ or ask jskowron@astrouw.edu.pl)

! 1 amp_bin_point_qs(mag0, source_cm, q, s)
! 2 amp_bin_point(zim, aim, Re_dsdzb, Im_dsdzb, jac, nimages, source, z1, z2, roots_guess, roots_guess_provided)
! 3 get_bin_poly_js_rcm_z(poly_, source, z1, z2)   - calculates 5th order  polynomial (binary lens equation) - in "reversed" center of mass

!-------------------------------------------------------------------!
! _1_                     AMP_BIN_POINT_QS                          !
!-------------------------------------------------------------------!
subroutine amp_bin_point_qs(mag0, source_cm, q, s)
  ! Jan Skowron
  use rkm
  implicit none

  ! CONSTANTS
  !integer, parameter :: RK=8  ! kind for real and complex variables

  ! FUNCTION ARGUMENTS
  real(kind=RK), intent(out) :: mag0
  complex(kind=RK), intent(in) :: source_cm
  real(kind=RK), intent(in) :: q, s

  !integer, intent(in) :: degree
  complex(kind=RK), dimension(degree) :: zim
  real(kind=RK), dimension(degree) :: aim
  real(kind=RK), dimension(degree) :: Re_dsdzb, Im_dsdzb, jac
  integer :: nimages
  complex(kind=RK) :: z1, z2, z1_, z2_, s_
  complex(kind=RK), dimension(degree) :: roots_guess
  logical :: roots_guess_provided
  real(kind=RK) :: m1, m2, offset_rcm
  complex(kind=RK) :: coffset_rcm
  integer :: i

  ! ------------- INIT VARS ----------------------------------

  m1 = 1d0/(1d0+q)
  m2 = q*m1
  !z1 = dcmplx(-m1*s0, 0d0)  ! m1*z2 + m2*z1 == 0 ('reversed' center of mass)
  !z2 = dcmplx(m2*s0, 0d0)
  z1 = dcmplx(-m2*s, 0d0)  ! center of mass is in 0,0
  z2 = dcmplx(m1*s, 0d0)

  offset_rcm = -s*(1d0-q)/(1d0+q)   ! z_CMS + coffset_rcm = z_rcm
  coffset_rcm = dcmplx(offset_rcm, 0d0)
  ! z1+coffset_rcm = -m2*s - s*(1d0-q)/(1d0+q) = -qs/(1d0+q)-s*(1d0-q)/(1d0+q) = -s/(1d0+q)
  ! z2+coffset_rcm =  m1*s - s*(1d0-q)/(1d0+q) =   s/(1d0+q)-s*(1d0-q)/(1d0+q) = qs/(1d0+q)

  ! ---------------- IMAGES ------------------------- 
  ! WARNING: all coordinates below are relative to "reversed" center of mass
  !          to facilitate faster ray shooting calculations
  !
  s_ =  source_cm + coffset_rcm
  z1_ = z1 + coffset_rcm
  z2_ = z2 + coffset_rcm

  roots_guess_provided = .false.
  roots_guess = dcmplx(0d0, 0d0)
  
  call amp_bin_point(zim, aim, Re_dsdzb, Im_dsdzb, jac, nimages, &
                     s_, z1_, z2_, roots_guess, roots_guess_provided)
  mag0 = 0d0
  !write (*,*) nimages
  do i = 1, nimages
    !write(*,*) i,  aim(i)
    mag0 = mag0 + dabs(aim(i))
  enddo
  return
end subroutine

!-------------------------------------------------------------------!
! _3_                       AMP_BIN_POINT                           !
!-------------------------------------------------------------------!
subroutine amp_bin_point(zim, aim, Re_dsdzb, Im_dsdzb, jac, nimages, &
                         source, z1, z2, roots_guess, roots_guess_provided)
  ! Jan Skowron
  use rkm
  implicit none

  ! CONSTANTS
  !integer, parameter :: RK=8  ! kind for real and complex variables

  ! FUNCTION ARGUMENTS
  !integer, intent(in) :: degree
  complex(kind=RK), dimension(degree), intent(inout) :: zim
  real(kind=RK), dimension(degree), intent(inout) :: aim
  real(kind=RK), dimension(degree), intent(out) :: Re_dsdzb, Im_dsdzb, jac
  integer, intent(out) :: nimages
  complex(kind=RK),intent(in) :: source
  complex(kind=RK), intent(in) :: z1, z2
  !real(kind=RK), intent(in) :: cost1, sint1, cost2, sint2
  complex(kind=RK), dimension(degree), intent(inout) :: roots_guess
  logical, intent(in) :: roots_guess_provided

  ! FOR CMPLX_ROOTS5
  complex(kind=RK), dimension(degree) :: roots
  logical :: first_3_roots_order_changed
  complex(kind=RK), dimension(degree+1)  :: poly
  logical, parameter :: polish_only = .false.

  ! TEMPORARY
  integer :: i
  complex(kind=RK) :: z_miss, z, z_bar
  real(kind=RK) :: ImA, ReA, amp, j, shortest2
  complex(kind=RK) :: a, b, ab, d, shortest, dsdzb

  if(roots_guess_provided) then
    roots = roots_guess
  else
    roots = dcmplx(0d0, 0d0)
  endif
  call get_bin_poly_js_rcm_z(poly, source, z1, z2)
  call cmplx_roots_5(roots, first_3_roots_order_changed, poly, polish_only)
  roots_guess = roots
  shortest = roots(4) - roots(5)
  shortest2 = dble(dconjg(shortest)*shortest)
  nimages = 0
  do i=1, degree
    z = roots(i)
    z_bar = dconjg(z)
    a = 1d0/(z_bar-z1)  ! z_bar - z1_bar
    b = 1d0/(z_bar-z2)  ! z_bar - z2_bar
    ab=a*b
    d = z_bar*ab  !  m1/(c-k) + m2/(c-l) == z_bar/((c-k)*(c-l))
    z_miss = source + d - z
    if(4*dreal(dconjg(z_miss)*z_miss)<shortest2) then  ! real image
      dsdzb = d*(a+b)-ab ! m1/(c-k)**2 + m2/(c-l)**2
      ReA = dreal(dsdzb)
      ImA = dimag(dsdzb)
      !dsdzb = m1*a*a + m2*b*b
      j = 1d0 - ReA*ReA - ImA*ImA
      amp = 1d0/j
      !jump_sq_i = ( (-sint*(1-ReA) - cost*ImA)**2 + (-sint*ImA + cost*(1+ReA))**2 )*amp*amp  ! ~(dz/ds)*direction squared
      nimages = nimages + 1
      zim(nimages) = z ! save image position
      aim(nimages) = amp
      Re_dsdzb(nimages) = ReA
      Im_dsdzb(nimages) = ImA
      jac(nimages) = j
    endif
  enddo
end

!-------------------------------------------------------------------!
! _3_                 GET_BIN_POLY_JS_RCM_Z                         !
!-------------------------------------------------------------------!
subroutine get_bin_poly_js_rcm_z(poly, source, z1, z2)
  ! everything in coordinate system where
  ! m1*z2 + m2*z1 == 0 ('reversed' center of mass)
  ! and m1 + m2 == 1
  ! z1 and z2 mast have no imaginary part (masses lie on x-axis)
  !
  implicit none
  ! FUNCTION ARGUMENTS
  integer, parameter :: RK=8  ! kind for real and complex variables
  integer, parameter :: degree=5
  complex(kind=RK), dimension(degree+1), intent(out)  :: poly
  complex(kind=RK), intent(in) :: source, z1, z2
  complex(kind=RK) :: sb
  complex(kind=RK) :: a, b ! A = z^2 + az + b
  complex(kind=RK) :: sb0, sb1, sb2 ! sbO = A*sb + z
  complex(kind=RK) :: K0, K1, K2 ! KO = A*KK + z
  complex(kind=RK) :: L0, L1, L2 ! LO = A*LL + z
  complex(kind=RK) :: KL0, KL1, KL2, KL3, KL4 ! KL = KO*LO
  complex(kind=RK) :: As0, As1, As2, As3, As4 ! As = A*sbO
  complex(kind=RK) :: KK, LL

  ! (z-s)*KO*LO - A*sbO
  ! (z-s)*KL - As
  sb = dconjg(source)
  KK = sb - z1 ! s_bar - z1_bar
  LL = sb - z2 ! s_bar - z2_bar

  ! A = z^2 + az + b
  a = - (z1 + z2)
  b = z1*z2

  !A = z^2 + az + b
  !z^2 1
  !z^1 a
  !z^0 b

  ! KO = A*KK + z
  K2 = KK
  K1 = 1d0+KK*a
  K0 = KK*b
  ! LO = A*LL + z
  L2 = LL
  L1 = 1d0+LL*a
  L0 = LL*b
  ! sbO = A*sb + z
  sb2 = sb
  sb1 = 1d0+sb*a
  sb0 = sb*b
  ! As = A*sbO = b*sb0 + sb2*z**4 + z**3*(a*sb2 + sb1) + z**2*(a*sb1 + b*sb2 +
  ! sb0) + z*(a*sb0 + b*sb1)
  As4 = sb2
  As3 = a*sb2 + sb1
  As2 = b*sb2 + a*sb1 + sb0
  As1 = a*sb0 + b*sb1
  As0 = b*sb0
  ! KL = KO*LO = K0*L0 + K2*L2*z**4 + z**3*(K1*L2 + K2*L1) + z**2*(K0*L2 + K1*L1
  ! + K2*L0) + z*(K0*L1 + K1*L0)
  KL4 = K2*L2
  KL3 = K1*L2 + K2*L1
  KL2 = K0*L2 + K1*L1 + K2*L0
  KL1 = K0*L1 + K1*L0
  KL0 = K0*L0
  ! (z-s)*KL - As = As0 - KL0*s + KL4*z**5 + z**4*(As4 + KL3 - KL4*s) +
  ! z**3*(As3 + KL2 - KL3*s) + z**2*(As2 + KL1 - KL2*s) + z*(As1 + KL0 - KL1*s)
  poly(6) = KL4
  poly(5) = -As4 + KL3 - KL4*source
  poly(4) = -As3 + KL2 - KL3*source
  poly(3) = -As2 + KL1 - KL2*source
  poly(2) = -As1 + KL0 - KL1*source
  poly(1) = -As0 - KL0*source
end

!-------------------------------------------------------------------!
! _4_                      AMP_BIN_LOOP_QS                          !
!-------------------------------------------------------------------!
subroutine amp_bin_loop_qs(zim_loop, mag0_loop, n_loop, source_cm, rho, q, s, nsteps)
  ! Jan Skowron
  use rkm
  implicit none

  ! CONSTANTS
  !integer, parameter :: RK=8  ! kind for real and complex variables
  !integer, parameter :: degree=5 ! binary lens, fifth order polynomial

  ! FUNCTION ARGUMENTS
  integer, intent(in) :: nsteps
  complex(kind=RK), intent(out), dimension(nsteps,degree) :: zim_loop
  real(kind=RK), intent(out), dimension(nsteps) :: mag0_loop
  integer, intent(out), dimension(nsteps) :: n_loop
  complex(kind=RK), intent(in) :: source_cm
  real(kind=RK), intent(in) :: rho, q, s

  complex(kind=RK), dimension(degree) :: zim
  real(kind=RK), dimension(degree) :: aim
  real(kind=RK) :: mag0
  real(kind=RK), dimension(degree) :: Re_dsdzb, Im_dsdzb, jac
  integer :: nimages
  complex(kind=RK) :: z1, z2, z1_, z2_, s_
  complex(kind=RK), dimension(degree) :: roots_guess
  logical :: roots_guess_provided
  real(kind=RK) :: m1, m2, offset_rcm
  complex(kind=RK) :: coffset_rcm, source
  integer :: i, k, nsteps_
  real(kind=RK), parameter :: twopi = 6.283185307179586d0
  real(kind=RK) :: phi_step, phi, rho_

  ! ------------- INIT VARS ----------------------------------

  m1 = 1d0/(1d0+q)
  m2 = q*m1
  !z1 = dcmplx(-m1*s0, 0d0)  ! m1*z2 + m2*z1 == 0 ('reversed' center of mass)
  !z2 = dcmplx(m2*s0, 0d0)
  z1 = dcmplx(-m2*s, 0d0)  ! center of mass is in 0,0
  z2 = dcmplx(m1*s, 0d0)

  offset_rcm = -s*(1d0-q)/(1d0+q)   ! z_CMS + coffset_rcm = z_rcm
  coffset_rcm = dcmplx(offset_rcm, 0d0)
  ! z1+coffset_rcm = -m2*s - s*(1d0-q)/(1d0+q) = -qs/(1d0+q)-s*(1d0-q)/(1d0+q) = -s/(1d0+q)
  ! z2+coffset_rcm =  m1*s - s*(1d0-q)/(1d0+q) =   s/(1d0+q)-s*(1d0-q)/(1d0+q) = qs/(1d0+q)

  ! ---------------- IMAGES ------------------------- 
  ! WARNING: all coordinates below are relative to "reversed" center of mass
  !          to facilitate faster ray shooting calculations
  !
  s_ =  source_cm + coffset_rcm
  z1_ = z1 + coffset_rcm
  z2_ = z2 + coffset_rcm

  roots_guess_provided = .false.
  roots_guess = dcmplx(0d0, 0d0)
  n_loop = 0
  nsteps_ = nsteps
  rho_ = rho
  if(rho<=0d0) then 
    nsteps_ = 1
    rho_ = 0d0
    zim = dcmplx(0d0, 0d0)
  endif
  phi_step = twopi/real(nsteps_)

  do k=1, nsteps_
    phi = (k-1)*phi_step
    source = s_ + rho_*exp(cmplx(0d0, phi))
    call amp_bin_point(zim, aim, Re_dsdzb, Im_dsdzb, jac, nimages, &
                       source, z1_, z2_, roots_guess, roots_guess_provided)
    mag0 = 0d0
    do i = 1, nimages ! loop along real images
      mag0 = mag0 + dabs(aim(i))
    enddo
    mag0_loop(k) = mag0
    zim_loop(k,:) = zim
    n_loop(k) = nimages
    roots_guess_provided = .true.
  enddo
  return
end subroutine


!-------------------------------------------------------------------!
