#!/usr/bin/env python

# Jan Skowron, 2010, 2016
# For educational use
# jskowron@astrouw.edu.pl
# http://www.astrouw.edu.pl/~jskowron/ulens/

######## IMPORTS ######################################################################################################
import sys,os
common_files=''
#common_files=os.path.join('.','ulens-common-files')  # requires two modules, they can be found in ../ulens-common-files
#sys.path.append('.')
try:
  import pygtk
  pygtk.require('2.0')
  import gtk
except ImportError, e:
  print 'Some modules for GUI required:'
  print     'Please install pygtk (eg. pygtk2 package)'
  print e
  sys.exit(1)
  
try:
  from numpy import *
except ImportError, e:
  print 'Some modules required:'
  print '    Please install numpy (eg. python2-numpy package)'
  print e
  sys.exit(1)

try:
  import caustic_js
except ImportError, e:
  print 'Some modules required:'
  print '    Please create module caustic_js.so'
  print 'run "./compile" in the source directory'
  print e
  sys.exit(1)

try:
  import bin_point_js
except ImportError, e:
  print 'Some modules required:'
  print '    Please create module bin_point_js.so'
  print 'run "./compile" in the source directory'
  print e
  sys.exit(1)

try:
  import matplotlib.pyplot as plt
  from matplotlib.figure import Figure
  from matplotlib.patches import Circle
except ImportError, e:
  print 'Some modules required:'
  print '    Please install module matplotlib (eg. python2-matplotlib package)'
  print e
  sys.exit(1)

try:
  from matplotlib.backends.backend_gtkagg import FigureCanvasGTKAgg as FigureCanvas
  from matplotlib.backends.backend_gtkagg import NavigationToolbar2GTKAgg
except ImportError, e:
  print 'Some modules required:'
  print '    Please install GtkAgg backend for matplotlib (eg. python2-matplotlib-gtk package)'
  print e
  sys.exit(1)

_new_tooltip_api =  (gtk.pygtk_version[1] >= 12)

###### MAIN CLASS ########################################################################################################



class mainWin(gtk.Window):
  def __init__(self):
    #### CONSTUCTOR ###
    super(mainWin,self).__init__()
    #self.set_size_request(250, 150)  #- hard limit
    self.set_default_size(1000,700) # - soft limit
    #self.set_position(gtk.WIN_POS_CENTER)
    #self.modify_bg(gtk.STATE_NORMAL, gtk.gdk.Color(6400, 6400, 6440))
    self.set_title('ulens - binary lens images')


    #self.debug=True
    self.debug=False

    #### SET ICON ####
    pixbuf = self.render_icon(gtk.STOCK_CDROM, gtk.ICON_SIZE_MENU)
    self.set_icon(pixbuf)
    #try:
    #  self.set_icon_from_file("icon.png")
    #except Exception, e:
    #   print e.message
    #   sys.exit(1)
    
    #### Create an accelgroup and add it to the window
    self.accel_group = gtk.AccelGroup()
    self.add_accel_group(self.accel_group)
    #key, mod = gtk.accelerator_parse('W')  # eg. ord('F') and gtk.gdk.CONTROL_MASK
    #self.connect('exit', lambda x:gtk.main_quit())
    #self.add_accelerator('exit',self.accel_group, key,
    #                         mod, gtk.ACCEL_VISIBLE)

    #### Create the menu File
    menu=gtk.MenuBar()
    menu_file=gtk.Menu()
    menu_file_head=gtk.MenuItem('_File')
    #key, mod = gtk.accelerator_parse('F')  # eg. ord('F') and gtk.gdk.CONTROL_MASK
    # Now add the accelerator to the menu item. Note that since we created
    # the menu item with a label the AccelLabel is automatically setup to 
    # display the accelerators.
    #menu_file_head.add_accelerator('activate', self.accel_group, key,
    #                          mod, gtk.ACCEL_HIDDEN)
    menu_file_head.set_submenu(menu_file)
    item_quit = gtk.ImageMenuItem(gtk.STOCK_QUIT, self.accel_group)
    item_quit.connect('activate', gtk.main_quit)
    menu_file.append(item_quit)
    menu.append(menu_file_head)

    #### Menu Help
    menu_help=gtk.Menu()
    menu_help_head=gtk.MenuItem('_Help')
    menu_help_head.set_submenu(menu_help)
    item_about = gtk.ImageMenuItem(gtk.STOCK_ABOUT, self.accel_group)
    item_about.connect('activate', self.on_about)
    menu_help.append(item_about)
    menu.append(menu_help_head)

    


    #### Sidebar
    sidebar=gtk.VBox(False,2)

    #### update button
    update_button=gtk.Button('_Update')
    update_button.connect('clicked',self.on_update)
    key, mod = gtk.accelerator_parse('<Control>R')  # eg. ord('F') and gtk.gdk.CONTROL_MASK
    update_button.add_accelerator('clicked',self.accel_group, key, mod, gtk.ACCEL_VISIBLE)

    ### alignment - fill bottom
    exit_button_al=gtk.Alignment(1,1,0,0)  # xalign=0.0, yalign=0.0, xscale=0.0, yscale=0.0

    ### exit row
    exit_row_box=gtk.HBox(False,3)           # homogeneous=False, spacing=0
    exit_button_al.add(exit_row_box)

    ### fix aspect ratio
    self.aspect_button=gtk.ToggleButton('Fixed aspect')
    self.aspect_button.set_active(True)
    self.aspect_button.connect('toggled',self.on_fix_aspect_ratio)
    exit_row_box.pack_start(self.aspect_button, False, True, 0) # expand=True, fill=True, padding=0

    #### exit button
    exit_button=gtk.Button('Close')
    exit_button.connect('clicked',gtk.main_quit)
    exit_row_box.pack_start(exit_button, False, True, 5)     # expand=True, fill=True, padding=0
    key, mod = gtk.accelerator_parse('<Control>W')  # eg. ord('F') and gtk.gdk.CONTROL_MASK
    exit_button.add_accelerator('clicked',self.accel_group, key, mod, gtk.ACCEL_VISIBLE)

    #### value_fields
    self.w_s=value_field('s=',0.7)
    self.w_q=value_field('q=',0.5)
    self.w_rho=value_field('rho=',0.05)
    #self.w_lgamma=value_field('Gamma=',0.3)
    self.w_x=value_field('x=',0.12)
    self.w_y=value_field('y=',0.06)

    #### images accuracy
    self.fstol = 0.001
    self.nsegments=500      # source segmentd
    self.nsteps_caustic = 400 # caustic segments
    self.nsteps_image = 1000 # image perimeter points count
    self.loops=[]



    #### matplotlib default params
    plt.rcParams['xtick.labelsize']='x-small'
    plt.rcParams['ytick.labelsize']='x-small'
    plt.rcParams['xtick.direction']='in'
    plt.rcParams['ytick.direction']='in'
    #plt.rcParams['']=''

    #### FIG1 ####
    self.fig = Figure(figsize=(8,6.5), dpi=100,facecolor=None)
    #ax = self.fig.add_subplot(111)
    #ax.plot([1,2,3])
    self.ax=self.fig.add_axes([0.08,0.05,1.0-0.08-0.02,1.0-0.05-0.02], axisbg='w') # [left, bottom, width, height]
    self.ax.axes.set_aspect('equal', adjustable='datalim', anchor='C')
    
    #### delete offsets on axis
    xtickFormatter=plt.ScalarFormatter(useOffset=False)
    xtickFormatter.set_scientific(True)
    xtickFormatter.set_powerlimits((-3,4))
    self.ax.xaxis.set_major_formatter(xtickFormatter)
    #with 1.0 or later try the following instead:
    #ticklabel_format(useOffset=False)
    ytickFormatter=plt.ScalarFormatter(useOffset=False)
    ytickFormatter.set_scientific(True)
    ytickFormatter.set_powerlimits((-3,4))
    self.ax.yaxis.set_major_formatter(ytickFormatter)

    #### drawing
    self.draw_lens()
    self.draw_source()
    self.ax.axis([-1.0,1.0,-1.0,1.0])
 
    self.canvas = FigureCanvas(self.fig)  # a gtk.DrawingArea

    #self.vpaned=gtk.VPaned()
    #self.vpaned.add1(self.canvas1)
    #self.vpaned.add2(self.canvas2)
    #self.vpaned.set_position(350)

    #### Toolbar
    self.toolbar = MyNavigationToolbar(self.canvas, self, os.path.join(common_files,'source-mover.png'))
    #toolbar.set_orientation(gtk.ORIENTATION_VERTICAL) - bad, label is wide

    #### label magnification
    self.label_mag=gtk.Label('mag=1')

    #### Fill sidebar
    sidebar.pack_start(menu,False, False, 0) # expand=True, fill=True, padding=0
    sidebar.pack_start(self.w_s,False,False,0)
    sidebar.pack_start(self.w_q,False,False,0)
    sidebar.pack_start(self.w_rho,False,False,0)
    #sidebar.pack_start(self.w_lgamma,False,False,0)
    sidebar.pack_start(self.w_x,False,False,0)
    sidebar.pack_start(self.w_y,False,False,0)
    #sidebar.pack_start(self.w_,False,False,0)
    sidebar.pack_start(self.label_mag,False,False,0)
    sidebar.pack_start(update_button,False,False,0)
    sidebar.pack_start(exit_button_al,True,True,0)

    #### box2
    vbox2=gtk.VBox(False,0)
    vbox2.pack_start(self.canvas,True, True, 0) # expand=True, fill=True, padding=0
    vbox2.pack_start(self.toolbar,False, False, 0) # expand=True, fill=True, padding=0

    #### Hbox
    hbox=gtk.HBox(False, 2)          # homogeneous=False, spacing=0
    hbox.pack_start(vbox2,True, True, 0) # expand=True, fill=True, padding=0
    hbox.pack_start(sidebar,False, False, 0) # expand=True, fill=True, padding=0

    #### Vbox
    #vbox=gtk.VBox(False, 2)          # homogeneous=False, spacing=0
    #vbox.pack_start(menu,False, False, 0) # expand=True, fill=True, padding=0
    #vbox.pack_start(hbox,True,True, 0)

    self.add(hbox)
    self.connect('destroy', gtk.main_quit)
    self.show_all()

    # color change
    #### colors
    bg_gtk_gdk_Color=self.get_style().bg[gtk.STATE_NORMAL]
    background_color=(bg_gtk_gdk_Color.red/65535.0, bg_gtk_gdk_Color.green/65535.0, bg_gtk_gdk_Color.blue/65535.0)
    #if self.debug: print self.get_style().bg.__getitem__(gtk.STATE_NORMAL)
    self.fig.set_facecolor(background_color)

    #### matplotlib events
    self.canvas.mpl_connect('pick_event',self.on_picked)
    self.canvas.mpl_connect('button_press_event',self.on_canvas_clicked)
   
    #### gtk events
    self.w_s.entry.connect('activate',self.on_update_lens)
    self.w_q.entry.connect('activate',self.on_update_lens)
    self.w_x.entry.connect('activate',self.on_update_source)
    self.w_y.entry.connect('activate',self.on_update_source)
    self.w_rho.entry.connect('activate',self.on_update_source)
    #self.w_lgamma.entry.connect('activate',self.on_update_source)


    #### images 
    self.update_images()


    #### end of constructor

  def on_about(self, widget):
    about = gtk.AboutDialog()
    about.set_name("ulens - binary lens images")
    about.set_version("1.0")
    about.set_copyright("Jan Skowron (jskowron@astrouw.edu.pl)")
    about.set_comments("For educational use")
    about.set_website("http://www.astrouw.edu.pl/~jskowron/ulens/")
    pixbuf = self.render_icon(gtk.STOCK_CDROM, gtk.ICON_SIZE_MENU)
    about.set_logo(pixbuf)
    #about.set_logo(gtk.gdk.pixbuf_new_from_file("battery.png"))
    about.run()
    about.destroy()

  def on_update_source(self,event):
    xmin,xmax,ymin,ymax=self.ax.axis()
    if self.debug: print 'on_update_source'
    self.update_source()
    self.update_images()
    self.ax.axis([xmin,xmax,ymin,ymax])
    self.canvas.draw()

  def on_update_lens(self,event):
    xmin,xmax,ymin,ymax=self.ax.axis()
    self.update_lens()
    self.update_images()
    self.ax.axis([xmin,xmax,ymin,ymax])
    self.canvas.draw()

  def on_update(self,event):
    xmin,xmax,ymin,ymax=self.ax.axis()
    self.update_source()
    self.update_lens()
    self.update_images()
    self.ax.axis([xmin,xmax,ymin,ymax])
    self.canvas.draw()

  def on_fix_aspect_ratio(self,event):
    if self.aspect_button.get_active():
      self.ax.axes.set_aspect('equal', adjustable='datalim', anchor='C')
    else:
      self.ax.axes.set_aspect('auto', adjustable='datalim', anchor='C')
    self.canvas.draw()

  def draw_source(self):
    x,y,rho = self.w_x.get_value(), self.w_y.get_value(), self.w_rho.get_value()
    self.source = Circle((x,y), max(rho, 0.0001), lw=0, alpha=0.8, color='green', fill=True, picker=100)
    self.source_shadow, = self.ax.plot([x],[y],color='gray',mew=0, marker='o',ms=6,alpha=0.25)
    self.ax.add_patch(self.source)
    #self.source=self.ax.scatter([x],[y],s=pi*rho**2,c='green',marker='o')
    #self.lens1=self.ax.plot([x1],[0],'ko',ms=max(2,10*m1))
    #self.lens2=self.ax.plot([x2],[0],'ko',ms=max(2,10*m2))

  def on_picked(self,event):
    artist=event.artist
    if isinstance(artist,Circle):
      if self.debug: print 'source picked'
      pass
    else:
      if self.debug: print 'something picked'
      pass

  def update_source(self):
    x,y,rho = self.w_x.get_value(), self.w_y.get_value(), self.w_rho.get_value()
    self.source.remove()
    self.source_shadow.remove()
    self.source=Circle((x,y), max(rho, 0.0001), lw=0, alpha=0.6, color='green', fill=True, picker=True)
    self.source_shadow,=self.ax.plot([x],[y],color='gray',mew=0, marker='o',ms=6,alpha=0.15)
    self.ax.add_patch(self.source)
    if self.debug: print 'update source'

  def draw_lens(self):
    s, q = self.w_s.get_value(), self.w_q.get_value()
    m1 = 1.0/(1.0+q)
    m2 =   q/(1.0+q)
    x1 =  -m2*s
    x2 =   m1*s
    #self.lenses=self.ax.scatter([x1,x2],[0,0],s=[m1,m2]*20,c='k',marker='o')
    self.lens1, = self.ax.plot([x1],[0],'ko',ms=max(2,10*m1))
    self.lens2, = self.ax.plot([x2],[0],'ko',ms=max(2,10*m2))
    self.ax.plot([0],[0],'k+',ms=4)
    ## caustic
    zca, zcr = caustic_js.get_bin_cau_js_qs(q, s, self.nsteps_caustic)
    self.critical_curve, self.caustic_curve = [], []
    for r in range(len(zca[0])):
      c,=self.ax.plot(real(zcr[:,r]),imag(zcr[:,r]),marker=',',ls='None',ms=1,mew=0,mfc='#87cefa')
      self.critical_curve.append(c)
      c,= self.ax.plot(real(zca[:,r]),imag(zca[:,r]),marker=',',ls='None',ms=1,mew=0,mfc='black')
      self.caustic_curve.append(c)

  def update_lens(self):
    s,q = self.w_s.get_value(), self.w_q.get_value()
    m1 =1.0/(1.0+q)
    m2 =  q/(1.0+q)
    x1 = -m2*s
    x2 =  m1*s
    #self.ax.scatter([x1,x2],[0,0],s=[m1,m2]*20,c='k',marker='o') 
    #self.lenses.
    self.lens1.set_data([[x1],[0]])
    self.lens2.set_data([[x2],[0]])
    #print 'update lens'
    ## caustic
    for c in self.critical_curve: c.remove()
    for c in self.caustic_curve: c.remove()
    zca, zcr = caustic_js.get_bin_cau_js_qs(q, s, self.nsteps_caustic)
    self.critical_curve, self.caustic_curve = [], []
    for r in range(len(zca[0])):
      c,=self.ax.plot(real(zcr[:,r]),imag(zcr[:,r]),marker=',',ls='None',ms=1,mew=0,mfc='#87cefa')
      self.critical_curve.append(c)
      c,= self.ax.plot(real(zca[:,r]),imag(zca[:,r]),marker=',',ls='None',ms=1,mew=0,mfc='black')
      self.caustic_curve.append(c)

  def on_canvas_clicked(self,event):
    # event.button, event.x, event.y, event.xdata, event.ydata
    #if self.debug: print 'on_canvas_click', event.button, event.xdata, event.ydata
    if event.inaxes==self.ax and event.button==2: # we are inside axes
      self.w_x.set_value(event.xdata)
      self.w_y.set_value(event.ydata)
      if self.debug: print 'on_canvas_click in axes with middle click'
      self.on_update_source(event)
    if event.inaxes==self.ax and event.button==1 and self.toolbar.mode=='source move':
      if self.debug: print 'on_canvas_click: left button, sourve move'
      self.w_x.set_value(event.xdata)
      self.w_y.set_value(event.ydata)
      self.on_update_source(event)
    #if self.debug: print self.get_style().bg.__getitem__(gtk.STATE_NORMAL)


  def update_images(self):
    color='r'
    x, y, rho = self.w_x.get_value(), self.w_y.get_value(), self.w_rho.get_value()
    s, q = self.w_s.get_value(), self.w_q.get_value()

    # center magnification
    zim_loop, mag0_loop, n_loop = bin_point_js.amp_bin_loop_qs(complex(x, y), 0.0, q, s, self.nsteps_image)
    if self.debug: 
      print 'update_images', mag0_loop[0]
    self.label_mag.set_text('mag_center=%.3e'%mag0_loop[0])

    for loop in self.loops: loop.remove()
    self.loops = []

    if rho>0.0:
      # loop
      zim_loop, mag0_loop, n_loop = bin_point_js.amp_bin_loop_qs(complex(x, y), rho, q, s, self.nsteps_image)

      #loop_x, loop_y = [], []
      #for l,length in enumerate(looplen):
      #  if length==0: continue
      #  # first and last points are the same (from getmag)
      #  lx,ly = loopx.T[l][0:length], loopy.T[l][0:length] 
      #  loop_x.append(lx)
      #  loop_y.append(ly)
      #  #parity.append(loopp[l])

      # test parity and direction
      #loop_directions = self.loops_add_or_substract(loop_x,loop_y)


      #for dir,lx,ly in zip(loop_directions,loop_x,loop_y):
      #  if dir==1: color='r'
      #  else: color='#b22222'  #'firebrick'
      idx = n_loop.argsort()[::-1]
      lengths = sort(unique(n_loop[idx]))[::-1]
      #nloops = len(lengths)

      #lps = []
      for i_ in range(5):
        npts = (n_loop > i_).sum()
        #print i_, npts
        if npts > 0:
          pts = zim_loop[idx][0:npts, i_]

          #for lenght_, idx_ in zip(lengths, idx):
          #  for i_ in range(lenght_):
          #    l[i_].append(zim_loop[idx_][i_])
          #for i_ in range(max(lengths)):

          loop,=self.ax.plot(real(pts), imag(pts), '.', color=color, ms=0.5)
          self.loops.append(loop)
    else:
      #for i in range(n_loop[0]):
      loop,=self.ax.plot(real(zim_loop[0][0:n_loop[0]]), imag(zim_loop[0][0:n_loop[0]]), '.', color=color, ms=4)
      self.loops.append(loop)


  def loops_add_or_substract(self,loop_x,loop_y):
    # make all positive loops counter-clockwise
    # and negative loops clockwise
    #
    #print 'update_images'
    #tot_area=0
    loop_dir = []
    for lx,ly in zip(loop_x,loop_y):
      area = -(lx[1:]*ly[:-1] - ly[1:]*lx[:-1]).sum()*0.5  # array have to have begin==end
      # area>0  <=>  counter-clockwise, this image should be added
      loop_dir.append(sign(area))       # remember current state

    loops_status=[]
    #c=['k','g','b','m','y']
    #print '------------------'
    for l,(lx,ly) in enumerate(zip(loop_x,loop_y)):
      loop_len=len(lx)
      if loop_len<4: continue
      #print par, area  
      #tot_area+=area 

      mid_poi=loop_len/2
      p1 = (array([lx[0], ly[0]])+array([lx[1], ly[1]]))*0.5
      p2 = (array([lx[mid_poi], ly[mid_poi]])+array([lx[mid_poi+1], ly[mid_poi+1]]))*0.5 # choose some points along the loop
      #self.ax.plot([p1[0],p2[0]], [p1[1],p2[1]], c=c[l])
      loops_status_i=self.test_all_loops_against_line(loop_x,loop_y,loop_dir,p1,p2) # loops plus line
      #debug print loops_status_i
      loops_status.append(loops_status_i)
    #print 'total area=',tot_area
    #print 'flat mag=',tot_area/pi/(self.w_rho.get_value())**2  # remember getmag returns 1.02 bigger loops
    #loop_dir_swap.append(1)           # 1 - keep directio, -1 - reverse direction
    loops_status=array(loops_status)
    loops_status_merge=[]
    for st in loops_status.T:
      fil=(st!=0)
      if fil.sum()==0: 
        print 'ERROR in loops_add_or_substract: empty'
        return loop_dir
      stf=st[fil]
      if (stf[0]==stf).all(): loops_status_merge.append(stf[0])
      else: 
        print 'ERROR in loops_add_or_substract: not match'
        return loop_dir
    if self.debug: print 'loops_status_merge', loops_status_merge

    ## perform reverse of direction
    for l,(status_cur, status_need) in enumerate(zip(loop_dir,loops_status_merge)):
      #debug print status_cur, status_need
      if status_cur!=status_need: # reverse image
        index=range(len(loop_x[l])-1,-1,-1)
        loop_x[l]=(loop_x[l])[index]
        loop_y[l]=(loop_y[l])[index]
      else:  
        # everything if ok, already good direction
        pass
    return loop_dir

  def test_all_loops_against_line(self,loop_x,loop_y,loop_dir,p1,p2):
    cross=(lambda u,v: (u[0]*v[1]-u[1]*v[0])) 
    u=p2-p1 # direction of line
    crosses , crossest = [] , []
    for l,(lx,ly) in enumerate(zip(loop_x,loop_y)):
      for l1,l2 in zip(c_[lx[:-1],ly[:-1]], c_[lx[1:],ly[1:]]):
        lu=l2-l1 # direction of segment
        #print l1,l2,lu,u,p1,p2
        cross_lu_u=cross(lu,u)
        if cross_lu_u!=0:
          s=(cross(p1,u)-cross(l1,u))/cross_lu_u
        else:
          continue
        if (s>=0.0) and (s<1.0): # line crosses segment
          t=-(cross(l1,lu)-cross(p1,lu))/cross_lu_u
          crossest.append(t)
          crosses.append(l)
        pass
      #print l, p1,p2, crosses
    result=array(loop_dir)*0
    crosses=array(crosses)[argsort(crossest)]
    # debug print l,crosses
    if len(crosses)==0: return result
    if (crosses==crosses[0]).all(): 
      result[crosses[0]]=1
      return result
    cur=crosses[0]
    st=0
    sig=1
    for c in crosses.tolist()+[50]:
      if c==cur: st+=1
      else:
        if st%2==0: 
          result[cur]=sig
          st=1
        else: 
          sig=-sig
          st=1
      if c==50: result[cur]=sig
      cur=c
    return result
    #return array(loop_dir)*0 # something is wrong

#### class #########################
class MyNavigationToolbar(NavigationToolbar2GTKAgg):
  def __init__(self, canvas, window, icon_file):
     NavigationToolbar2GTKAgg.__init__(self, canvas, window)
     #(('Home', 'Reset original view', 'home.png', 'home'),
     #('Back', 'Back to  previous view', 'back.png', 'back'),
     #('Forward', 'Forward to next view', 'forward.png', 'forward'),
     #('Pan', 'Pan axes with left mouse, zoom with right', 'move.png', 'pan'),
     #('Zoom', 'Zoom to rectangle', 'zoom_to_rect.png', 'zoom'),
     #(None, None, None, None),
     #('Subplots', 'Configure subplots', 'subplots.png', 'configure_subplots'),
     #('Save', 'Save the figure', 'filesave.png', 'save_figure'))

     # self.message is a gtk.Label()

     #self.pan()
     self.mode='source move'

     toremove=[]
     home=None
     for item in self:
       if hasattr(item,'get_label'):
         label=item.get_label()
         if label=='Home':home=item
         if label in ['Home','Forward','Subplots']:
           #print item.set_visible(False)
           toremove.append(item)
     for item in toremove: self.remove(item)
     if home!=None: self.insert(home, self.get_n_items())

     # this is copied from /usr/lib64/python2.4/site-packages/matplotlib/backends/backend_gtk.py
     icon=gtk.Image()
     icon.set_from_file(icon_file)
     tbutton = gtk.ToolButton(icon, 'SourceMover')
     self.insert(tbutton, min(3,self.get_n_items()))
     callback='reset_pan_zoom_state'
     tbutton.connect('clicked', getattr(self, callback))
     tooltip_text='Move the source'
     if not _new_tooltip_api:
       #self.tooltips = gtk.Tooltips()
       tbutton.set_tooltip(self.tooltips, tooltip_text, 'Private')
     else:
       tbutton.set_tooltip_text(tooltip_text)

  def reset_pan_zoom_state(self, event):
    # this is doen using info from: /usr/lib64/python2.4/site-packages/matplotlib/backend_bases.py
    if self._active=='PAN': self.pan()
    if self._active=='ZOOM': self.zoom()
    if self.mode=='': self.mode='source move'
    self.set_message(self.mode) 


#### class #########################
class value_field(gtk.HBox):
  def __init__(self,label_txt,init_value):
    super(gtk.HBox,self).__init__(homogeneous=False,spacing=0)
    label=gtk.Label(label_txt)
    self.entry=gtk.Entry()
    self.set_value(init_value)
    self.pack_start(label,False,False,0)
    self.pack_start(self.entry,True,True,0)
  
  def set_value(self,value):
    if isinstance(value,float):
      val=value
    elif isinstance(value,str):
      try:
        val=float(value)
      except ValueError:
        val=1.0
    else:
      val=1.0
    self.entry.set_text('%f'%val)

  def get_value(self):
    try:
      val=float(self.entry.get_text())
    except ValueError:
      val=1.0
    return val
 

#### main #########################
if __name__=='__main__':
  mainWin()
  gtk.main()


